/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "cube.h"

primitive_t * cubeCreate(float w, float h, float d) {
  primitive_t *cube = primitiveCreate(8, 36);

  vertice_t *vertices = malloc(sizeof(vertice_t) * cube->verticeCount);
  indice_t *indices = malloc(sizeof(indice_t) * cube->indiceCount);

  float hw = w / 2, hh = h / 2, hd = d / 2;
  vertices[0].x = -hw, vertices[0].y = -hh, vertices[0].z = -hd;
  vertices[0].u = 0, vertices[0].v = 0;

  vertices[1].x = hw, vertices[1].y = -hh, vertices[1].z = -hd;
  vertices[1].u = 1, vertices[1].v = 0;

  vertices[2].x = -hw, vertices[2].y = hh, vertices[2].z = -hd;
  vertices[2].u = 0, vertices[2].v = 1;

  vertices[3].x = hw, vertices[3].y = hh, vertices[3].z = -hd;
  vertices[3].u = 1, vertices[3].v = 1;

  vertices[4].x = -hw, vertices[4].y = -hh, vertices[4].z = hd;
  vertices[4].u = 0, vertices[4].v = 0;

  vertices[5].x = hw, vertices[5].y = -hh, vertices[5].z = hd;
  vertices[5].u = 1, vertices[5].v = 0;

  vertices[6].x = -hw, vertices[6].y = hh, vertices[6].z = hd;
  vertices[6].u = 0, vertices[6].v = 1;

  vertices[7].x = hw, vertices[7].y = hh, vertices[7].z = hd;
  vertices[7].u = 1, vertices[7].v = 1;

  // Back
  indices[ 0] = 0, indices[ 1] = 1, indices[ 2] = 3;
  indices[ 3] = 0, indices[ 4] = 2, indices[ 5] = 3;
  
  // Right
  indices[ 6] = 1, indices[ 7] = 5, indices[ 8] = 7;
  indices[ 9] = 1, indices[10] = 3, indices[11] = 7;

  // Left
  indices[12] = 4, indices[13] = 0, indices[14] = 2;
  indices[15] = 4, indices[16] = 6, indices[17] = 2;

  // Front
  indices[18] = 5, indices[19] = 4, indices[20] = 6;
  indices[21] = 5, indices[22] = 7, indices[23] = 6;

  // Top
  indices[24] = 7, indices[25] = 2, indices[26] = 6;
  indices[27] = 7, indices[28] = 3, indices[29] = 2;

  // Bottom
  indices[30] = 1, indices[31] = 0, indices[32] = 4;
  indices[33] = 1, indices[34] = 4, indices[35] = 5;


  primitiveBufferVertices(cube, 0, cube->verticeCount, vertices);
  primitiveBufferIndices(cube, 0, cube->indiceCount, indices);
  free(vertices);
  free(indices);

  return cube;
}